import signal
import subprocess

import click
from rich.text import Text
from .exportdata import export
from .helpers import connectioncheck, signal_handler, first_run, seat_check, show_panel
from .parsepdf import pdf_to_df
from .savefile import save_all


@click.command()
@click.option(
    "--file",
    prompt=click.style("PDF file", fg="cyan"),
    help="File containg name, seatno and mothername of students.",
)
@click.option(
    "--semester",
    default="I",
    prompt=click.style("Select semester", fg="cyan"),
    type=click.Choice(["I", "II", "III", "IV"], case_sensitive=False),
    help="Result semester.",
)
@click.option(
    "--sseat",
    prompt=click.style("Start seat no.(0 - starts from top)", fg="cyan"),
    type=click.types.INT,
    default=0,
    show_default=False,
    help="Starting seat no of the given semester.",
)
@click.option(
    "--eseat",
    prompt=click.style("End seat no.(0 - stops at last)", fg="cyan"),
    type=click.types.INT,
    default=0,
    show_default=False,
    help="Ending seat no of the given semester.",
)
@click.option(
    "--outfile",
    prompt=click.style("Output file name", fg="cyan"),
    default="marks-[semsester].xlsx",
    show_default=False,
    help="Excel file name to which data will be written.",
)
def main(file: str, semester: str, sseat: int, eseat: int, outfile: str):
    data = pdf_to_df(file)

    # check if seat no are valid
    seat_check(data, sseat, eseat)

    # clear the screen before proceeding
    click.clear()

    # save all html file for rows in data
    _ = save_all(data)

    # wait for user to exit
    click.pause()

    # export marks to excel file
    file_path = export(data, sseat, eseat, semester, outfile)

    # wait for user to exit
    click.pause()

    # launch explorer where file has been stored
    click.launch(file_path, locate=True)


def run():
    # register signal handler to handle abrupt execution
    signal.signal(signal.SIGINT, signal_handler)

    # if the system has no internet, halt execution
    connectioncheck()

    # if this is first run of application, run command 'playwright install'
    if first_run():
        click.echo("Hey! Need to install few things. Hold on...")
        status = subprocess.run(["playwright", "install"])
        if not status.returncode == 0:
            raise SystemExit("Something went wrong! Try running `playwright install`")
        click.clear()

    show_panel()

    while True:
        # main entry function
        main()


if __name__ == "__main__":
    run()



import os
import pathlib
from collections import deque

import pandas as pd
from playwright.sync_api import sync_playwright
from rich.align import Align
from rich.console import Group
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.progress import (BarColumn, MofNCompleteColumn, Progress,
                           SpinnerColumn, TimeElapsedColumn)
from rich.text import Text
from .helpers import dynamic_table

__FOLDER = os.path.join(pathlib.Path.home(), ".ezeeresult", "html")


def __tofile(folder: str, filename: str, content: str):
    """
    Helper function to write content to files. Creates folders if they don't exist
    """
    if not os.path.exists(folder):
        os.makedirs(folder)
    with open(f"{folder}/{filename}", 'w') as f:
        f.write(content)


def save_all(data: pd.DataFrame):

    # region rich layout

    progress = Progress(
        "{task.description}",
        BarColumn(),
        SpinnerColumn(),
        MofNCompleteColumn(),
        TimeElapsedColumn(),
    )

    download_progress = progress.add_task(
        "[cyan]Downloading...",
        total=len(data.index)
    )

    # Create a layout and split it into three sections
    layout = Layout("root")
    layout.split(
        Layout(name="top", ratio=5),
        Layout(name="bottom", ratio=1),
    )

    layout['bottom'].split_row(
        Layout(name="bottom-left"),
        Layout(name="bottom-right")
    )

    layout['bottom-left'].update(
        Panel(
            Align.center(progress, vertical="middle"),
            title="Progress",
            border_style="green"
        )
    )

    layout['bottom-right'].update(
        Panel(
            Align.center(
                "[bold green]Downloading...[/bold green]", vertical="middle"),
            title="Status",
            border_style="green",
            title_align="center"
        )
    )

    columns = ["Seat No", "Name", "Status"]
    _, height = os.get_terminal_size()
    rows = deque(maxlen=height-15)

    layout['top'].update(
        Panel(
            dynamic_table(rows, columns),
            title="Saved Files",
            border_style="green",
            padding=1,
        ))

    # endregion

    with sync_playwright() as playwright:

        # Create a browswer instance and goto url
        browser = playwright.chromium.launch()
        context = browser.new_context()
        page = context.new_page()

        page.goto("https://results.unipune.ac.in/")

        # Once on page, get the first row having MASTER OF COMPUTER APPLICATIONS (REVISED 2020) as its td.
        # Select the first row from rows, which ensures you are always getting latest results
        page.get_by_role("row", name=r'MASTER OF COMPUTER APPLICATIONS \(REVISED 2020\)').first.get_by_role(
            "button", name="Go for Result").click()

        # rich live view
        with Live(layout, refresh_per_second=4, vertical_overflow="visible") as live:
            for x in data.itertuples():
                # if the file already exists, add it to table, update the progress and loop over
                if os.path.exists(f"{__FOLDER}/{x.SeatNo}.html"):
                    rows.append((str(x.SeatNo), x.Student, "Exists"))
                    layout['top'].update(
                        Panel(
                            dynamic_table(rows, columns),
                            title="Saved Files",
                            border_style="green",
                            padding=1,
                        ))
                    progress.update(download_progress, advance=1)
                    continue

                page.type("#ctl00_ContentPlaceHolder1_txtSeatno",
                          str(x.SeatNo), delay=2)
                page.type("#ctl00_ContentPlaceHolder1_txtMother",
                          str(x.Mother), delay=2)
                page.click("#ctl00_ContentPlaceHolder1_btnSubmit",
                           delay=2)

                page.once("domcontentloaded",
                          lambda y: __tofile(
                              folder=__FOLDER,
                              filename=f'{x.SeatNo}.html',
                              content=y.content()
                          ))

                rows.append((str(x.SeatNo), x.Student, "Downloaded"))

                layout['top'].update(
                    Panel(
                        dynamic_table(rows, columns),
                        title="Saved Files",
                        border_style="green",
                        padding=1,
                    ), )

                progress.advance(download_progress)
                page.fill("#ctl00_ContentPlaceHolder1_txtSeatno", "")
                page.fill("#ctl00_ContentPlaceHolder1_txtMother", "")

            _, _, files = next(os.walk(__FOLDER))
            file_count = len(files)

            txt_top = Text.from_markup(
                f"[bold green]Downloaded {file_count} of {len(data.index)} files.[/bold green]", justify="center"
            )

            # txt_bot = Text.from_markup(
            #     f"[bold green]Pages saved at:[/bold green] [red]{__FOLDER}[/red]", justify="center")

            # txt_grp = Group(txt_top, txt_bot)

            layout['bottom-right'].update(
                Panel(
                    Align.center(
                        txt_top, vertical="middle"),
                    title="Status",
                    border_style="green",
                    title_align="center"
                )
            )

    if not file_count == len(data.index):
        progress.update(download_progress, description="Verifying...")
        save_all(data)
    return f'{pathlib.Path()}/{__FOLDER}'


def save_one(seat_no: str, mother: str, student: str):
    '''Save a page corresponding to info at given url for later parsing'''

    with sync_playwright() as playwright:

        browser = playwright.chromium.launch()
        context = browser.new_context()

        page = context.new_page()
        page.goto("https://results.unipune.ac.in/")

        page.get_by_role("row",
                         name=r'MASTER OF COMPUTER APPLICATIONS \(REVISED 2020\)'
                         ).first.get_by_role("button",
                                             name="Go for Result").click()

        page.type("#ctl00_ContentPlaceHolder1_txtSeatno",
                  str(seat_no), delay=2)
        page.type("#ctl00_ContentPlaceHolder1_txtMother",
                  str(mother), delay=2)
        page.click("#ctl00_ContentPlaceHolder1_btnSubmit",
                   delay=2)

        page.once("domcontentloaded",
                  lambda y: __tofile(
                      folder=__FOLDER,
                      filename=f'{seat_no}.html',
                      content=y.content()
                  ))



import numpy as np
import pandas as pd
import tabula
from rich import print
from rich.progress import (BarColumn, Progress, SpinnerColumn, TextColumn,
                           TimeElapsedColumn)


def pdf_to_df(filename: str) -> pd.DataFrame:
    """"Read file containing seatno, student name, mothers name and prn and create dataframe from parsed data."""

    progress = Progress(
        TextColumn("{task.description}"),
        BarColumn(),
        SpinnerColumn(),
        TimeElapsedColumn()
    )

    master_task = progress.add_task("[cyan] Processing file...", total=4)

    try:
        with progress:
            while not progress.finished:

                # Using tabula read all the pages containing tables and store tables in numpy array
                pdf_pages = tabula.io.read_pdf(filename, pages='all')
                pages_arr = np.array(pdf_pages, dtype=object)

                progress.update(master_task, advance=1,
                                description="[cyan]Reading file...")

                # Convert each table into pandas dataframe
                table_dfs = list(map(lambda x: pd.DataFrame(x), pages_arr))

                progress.update(master_task, advance=1,
                                description="[cyan]Parsing file...")

                # remove '\r' from the parsed tables
                for x in table_dfs:
                    x.replace('\\r', ' ', regex=True, inplace=True)

                progress.update(master_task, advance=1,
                                description="[cyan]Cleaning dataframe...")

                combined_tabels = pd.concat(table_dfs)
                combined_tabels.rename(
                    columns={'Student Name': 'Student'},
                    inplace=True)

                # if the columns of dataframe don't contain required fields, print error and exit
                if not set(['SeatNo', 'Mother', 'Student']).issubset(combined_tabels.columns):
                    print(
                        "[bold red]Tables not found! Try again with correct file.[/bold red]")
                    raise SystemExit()

                progress.update(master_task, advance=1,
                                description="[cyan]Finished!")
    except Exception as e:
        progress.update(master_task, advance=4, description="[red] Error...")
        print("[red bold]Invalid PDF file! Try again with correct file.[/bold red]")
        raise SystemExit()

    return combined_tabels



import os
import pathlib
import socket
import sys
from collections import deque
from enum import Enum

import requests
import pandas as pd
from rich import box, print
from rich.console import Console
from rich.layout import Layout
from rich.table import Table
from rich.text import Text
from rich.panel import Panel


def show_panel():
    txt = Text.from_markup(
        "[bold green]Result Scraper \n [red]MASTER OF COMPUTER APPLICATIONS (REVISED 2020)[/red][/bold green]"
    )
    txt.justify = "center"

    panel = Panel(txt)
    panel.title_align = "center"
    print(panel)


def first_run() -> bool:
    """
    Function to check if this is the application's first run.
    """
    __FOLDER = os.path.join(pathlib.Path.home(), ".ezeeresult")
    if os.path.exists(f"{__FOLDER}/.first"):
        return False
    else:
        try:
            os.makedirs(__FOLDER)
        except FileExistsError:
            with open(f"{__FOLDER}/.first", "w+") as f:
                f.write("Hey! you discovered me!")
            return True


def seat_check(std_data: pd.DataFrame, sseat: int, eseat: int):
    """
    Function to check if seat no is in valid range. And start seat no is not after end seat no.
    """
    min_seat = std_data["SeatNo"].min()
    max_seat = std_data["SeatNo"].max()

    if sseat and (sseat < min_seat or sseat > max_seat):
        raise SystemExit("Invalid Start SeatNo!")
    if eseat and (eseat < min_seat or eseat > max_seat):
        raise SystemExit("Invalid End SeatNo!")
    if (sseat and eseat) and (sseat > eseat):
        raise SystemExit("Invalid Seat No(s)!")


def static_table(rows: list) -> Table:
    """Simple table used to render information of top and bottom performers"""

    table = Table(box=box.SIMPLE, expand=True)
    table.add_column("Rank")
    table.add_column("SeatNo")
    table.add_column("Name")
    table.add_column("Marks")

    for i, row in enumerate(rows, start=1):
        table.add_row(str(i), *row)

    return table


def dynamic_table(rows: deque, columns: list = []):
    """Table having height of terminal and updates new values constantly"""

    layout = Layout()
    console = Console()

    table = Table(box=box.SIMPLE, expand=True)
    row_count = os.get_terminal_size()[1]

    rows = list(rows)
    while row_count >= 0:
        table = Table(box=box.SIMPLE, expand=True)
        for i in columns:
            table.add_column(i)

        for row in rows[-row_count:]:
            table.add_row(*row)

        layout.update(table)
        render_map = layout.render(console, console.options)

        if len(render_map[layout].render[-1]) > 2:
            row_count -= 1
        else:
            break

    return table


class InvalidHTMLFileError(Exception):
    """
    Error class generated when the file is invalid and can't be parsed
    """

    def __init__(self, *args: object) -> None:
        super().__init__(*args)


def connectioncheck():
    """
    Function to check if the system is connected to internet and can reach the url.
    """
    url = "https://results.unipune.ac.in"
    ip = socket.gethostbyname(socket.gethostname())

    if ip.startswith("127.0"):
        raise SystemExit(
            "Your system has no internet connection! Please try again later."
        )

    if requests.get(url).status_code != 200:
        raise SystemExit(f"'{url}' not reachable! Please try again later.")


def signal_handler(signal, frame):
    """
    Handles CTRL+C during program execution
    """

    print()
    print("Aborted! Exiting...")
    sys.exit(0)


class Semester(Enum):
    FIRST = 1
    SECOND = 2
    THIRD = 3
    FOURTH = 4

    @classmethod
    def get(cls, sem: str):
        match sem:
            case "I":
                return cls.FIRST
            case "II":
                return cls.SECOND
            case "II":
                return cls.THIRD
            case "IV":
                return cls.FOURTH
            case _:
                cls.FIRST

    def __str__(self):
        return str(self.value)


SUBJECTS: dict = {
    Semester.FIRST: {
        "JAVA": ["INT", "EXT"],
        "DSA": ["INT", "EXT"],
        "OOSE": ["INT", "EXT"],
        "OSC": ["INT", "EXT"],
        "NT": ["INT", "EXT"],
        "OC-I": ["INT", "EXT"],
        "OC-II": ["INT", "EXT"],
        "PRAC": ["INT", "EXT"],
        "MINI": ["INT", "EXT"],
        "SSKILL": ["INT", "EXT"],
        "HR-I": ["INT", "EXT"],
        "ISEC-I": ["INT", "EXT"],
    },
    Semester.SECOND: {
        "PYTHON": ["INT", "EXT"],
        "SPM": ["INT", "EXT"],
        "OT": ["INT", "EXT"],
        "AIT": ["INT", "EXT"],
        "ADBMS": ["INT", "EXT"],
        "OC-III": ["INT", "EXT"],
        "OC-IV": ["INT", "EXT"],
        "PRAC": ["INT", "EXT"],
        "MINI": ["INT", "EXT"],
        "SSKILL": ["INT", "EXT"],
        "HR-II": ["INT", "EXT"],
        "ISEC-II": ["INT"],
    },
    Semester.THIRD: {
        "MAD": ["INT", "EXT"],
        "DWDM": ["INT", "EXT"],
        "STQA": ["INT", "EXT"],
        "KRAI": ["INT", "EXT"],
        "CC": ["INT", "EXT"],
        "OC-V": ["INT", "EXT"],
        "OC-VI": ["INT", "EXT"],
        "PRAC": ["INT", "EXT"],
        "MINI": ["INT", "EXT"],
        "SSKILL": ["INT", "EXT"],
        "ISEC-III": ["INT", "EXT"],
        "SKD-I": ["INT", "EXT"],
        "ITC": ["INT", "EXT"],
    },
    Semester.FOURTH: {
        "DEVOPS": ["INT", "EXT"],
        "PPM&OB": ["INT", "EXT"],
        "PROJ": ["INT", "EXT"],
        "ISEC-IV": ["INT", "EXT"],
        "SKD-II": ["INT", "EXT"],
    },
}



import re

import click
import pandas as pd
from bs4 import BeautifulSoup
from .helpers import InvalidHTMLFileError, Semester


def __extract(file: str) -> list:
    """
    Parses the html file and extracts tables containing marks
    """

    soup = None

    try:
        with open(file, 'r') as f:
            contents = f.read()
            soup = BeautifulSoup(contents, 'html.parser')
        table = soup.findAll(lambda tag: tag.name == 'table')
        marks_table = table[1]
        rows = marks_table.findAll(lambda tag: tag.name == 'tr')

        store = list()

        for row in rows:
            store.append(re.findall(r'[\b|\s|\d](\d{1,}|\-|!)[\b]?', str(row)))

    # if the table doesn't exists, then index error is raised
    except IndexError:
        raise InvalidHTMLFileError("File seems to be corrupted!")

    """
    Remove empty list from marks table. 
    First 3 rows are student info, and last 5 rows are statistics of result
    """
    return list(filter(lambda x: len(x) > 0, store[3:-5]))


def __clean(store: list, semester: Semester) -> list:
    """
    Function to extract marks of the semster passed as argument
    """
    local_store = []
    indices = {
        Semester.FIRST: slice(0, 12),
        Semester.SECOND: slice(13, 25),
        Semester.THIRD: slice(26, 39),
        Semester.FOURTH: slice(40, 45)
    }

    try:
        match semester:
            case Semester.FIRST:
                local_store = store[indices.get(semester)]
                local_store[10].append('-')
                local_store[11].append('-')
                return [x[-5:-3] for x in local_store]

            case Semester.SECOND:
                local_store = store[indices.get(semester)]
                local_store[10].append('-')
                local_store[11].append('-')
                return [x[-5:-3] for x in local_store]

            case Semester.THIRD:
                local_store = store[indices.get(semester)]
                local_store[10].append('-')
                local_store[11].append('-')
                local_store[12].append('-')
                return [x[-5:-3] for x in local_store]
            case Semester.FOURTH:
                local_store = store[indices.get(semester)]
                local_store[3].append('-')
                local_store[4].append('-')
                return [x[-5:-3] for x in local_store]
            case _:
                local_store = store
                return local_store
    except IndexError:
        click.clear()
        raise SystemExit(
            f"Semester {semester} not found for the given roll(s)!")


def __convert(store: list, seat_no: int, s_name: str, semester: Semester) -> list:
    """
    Converts and fills the table with 0 and adds sum at end of the lists.
    """
    sem_i_iii = slice(0, 20)
    sem_iv = slice(0, 3)

    df = pd.DataFrame(store)
    df = df.apply(pd.to_numeric, errors='coerce').fillna(0)
    marks = df.to_numpy().flatten().tolist()

    if semester == Semester.FOURTH:
        s_marks = sum(marks[sem_iv])
        marks.append(s_marks)
    else:
        s_marks = sum(marks[sem_i_iii])
        marks.append(s_marks)

    marks.insert(0, s_name)
    marks.insert(0, seat_no)

    return marks


def parse(file: str, semester: Semester, seat: int, name: str) -> list:
    return __convert(__clean(__extract(file), semester), seat, name, semester)



import os
import pathlib
import time
from collections import deque
from itertools import chain

import pandas as pd
from openpyxl import Workbook
from openpyxl.utils.dataframe import dataframe_to_rows
from rich.align import Align
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.progress import (BarColumn, MofNCompleteColumn, Progress,
                           SpinnerColumn, TimeElapsedColumn)
from rich.text import Text

from .helpers import (SUBJECTS, InvalidHTMLFileError, Semester, dynamic_table,
                      static_table)
from .parsehtml import parse
from .savefile import save_one

__FOLDER = os.path.join(pathlib.Path.home(), ".ezeeresult", "html")


def export(df: pd.DataFrame, sseat: int, eseat: int, semester: str, filename: str):
    """
    Function to export collected values as excel file
    """
    data = None
    if sseat and eseat:
        data = df.query(f'SeatNo >= {sseat} and SeatNo <= {eseat}')
    elif sseat and not eseat:
        data = df.query(f'SeatNo >= {sseat}')
    elif eseat and not sseat:
        data = df.query(f'SeatNo <= {eseat}')
    else:
        data = df

    collected = []
    semester = Semester.get(semester)

    #! rich view start
    progress = Progress(
        "{task.description}",
        BarColumn(),
        SpinnerColumn(),
        MofNCompleteColumn(),
        TimeElapsedColumn(),
    )

    parsing_task = progress.add_task("[cyan]Extracting",
                                     total=len(data.index))

    layout = Layout("root")
    layout.split(Layout(name="top", ratio=5), Layout(name="bottom", ratio=1))

    layout['top'].split_row(
        Layout(name="top-left"),
        Layout(name="top-right")
    )

    layout['top-right'].split_column(
        Layout(name="top-right-top"),
        Layout(name="top-right-bottom"),
    )

    layout['top-right'].update(Panel("...", title="Toppers"))

    layout['bottom'].split_row(
        Layout(name="bottom-left"),
        Layout(name="bottom-right")
    )

    layout['bottom-left'].update(
        Panel(
            Align.center(progress, vertical="middle"),
            title="Progress",
            border_style="green",
            padding=(1),
        ))

    layout['bottom-right'].update(
        Panel(
            Align.center(
                "[bold green]Exporting...[/bold green]", vertical="middle"),
            title="Status",
            border_style="green",
            title_align="center"
        )
    )

    columns = ["Seat No", "Name", "PRN"]

    _, height = os.get_terminal_size()
    rows = deque(maxlen=height-15)
    #! rich view end

    top10 = []
    bot10 = []
    with Live(layout, refresh_per_second=4, vertical_overflow="visible") as live:
        for x in data.itertuples():
            try:

                parsed = parse(f'{__FOLDER}/{x.SeatNo}.html',
                               semester, x.SeatNo, x.Student)
                collected.append(parsed)

                rows.append((str(x.SeatNo), str(x.Student), str(x.PRN)))
                layout['top-left'].update(
                    Panel(
                        dynamic_table(rows, columns),
                        title="Saved Files",
                        border_style="green",
                    ))
                progress.update(parsing_task, advance=1)

            except InvalidHTMLFileError as e:
                itxt = Text.from_markup(
                    f"[bold green]Downloading [red]{x.SeatNo} | {x.Student}...[red] [/bold green]"
                )
                itxt.justify = 'center'

                layout['bottom-right'].update(
                    Panel(itxt, title=e or "Error",
                          border_style='green', )
                )

                save_one(str(x.SeatNo), x.Mother, x.Student)
                parsed = parse(f'{__FOLDER}/{x.SeatNo}.html',
                               semester, x.SeatNo, x.Student)

                collected.append(parsed)

                rows.append((str(x.SeatNo), str(x.Student), str(x.PRN)))

                layout['top-left'].update(
                    Panel(
                        dynamic_table(rows, columns=columns),
                        title="Saved Files",
                        border_style="green",
                    ))
                progress.update(parsing_task, advance=1)

            top10 = _top10s(collected)
            bot10 = _bot10s(collected)
            layout['top-right-top'].update(
                Panel(static_table(top10), title="Top Rankers", border_style="green",
                      )
            )
            layout['top-right-bottom'].update(
                Panel(static_table(bot10), title="Bottom Rankers", border_style="green",
                      )
            )
            time.sleep(0.1)
        collected = pd.DataFrame(collected)

        if not filename:
            filename = f"marks-{semester}.xlsx"
        elif filename == "marks-[semsester].xlsx":
            filename = f"marks-{semester}.xlsx"
        else:
            fname, ext = os.path.splitext(filename)
            if not ext:
                filename = f"{fname}.xlsx"
        file_path = _toexcel(collected, semester, filename)

        layout['bottom-right'].update(
            Panel(
                Align.center(
                    f"[bold green]Exported file: [red]{filename}[red][/bold green]", vertical="middle"),
                title="Exported",
                border_style="green",
                title_align="center"
            )
        )

    return file_path


def _top10s(data: list):
    temp = sorted(data, key=lambda x: x[-1], reverse=True)[:10]
    temp = list(map(lambda x: (str(x[0]), str(x[1]), str(x[-1])), temp))
    return temp


def _bot10s(data: list):
    temp = sorted(data, key=lambda x: x[-1])[:10]
    temp = list(map(lambda x: (str(x[0]), str(x[1]), str(x[-1])), temp))
    return temp


def _toexcel(data: pd.DataFrame, sem: Semester, filename: str):
    """
    Write dataframe to excel file
    """
    wb = Workbook()

    ws = wb.active
    ws.title = f'semester-{sem}'

    subs_dict: dict = SUBJECTS[sem]
    top_row: list = list(subs_dict.keys())
    top_row.insert(0, '')
    top_row.insert(1, '')

    sub_sec: list = list(subs_dict.values())

    next_row: list = list(chain(*sub_sec))
    next_row.append('TOTAL')
    next_row.insert(0, 'SEAT')
    next_row.insert(1, 'NAME')

    for i in range(3, len(top_row)*2, 2):
        top_row.insert(i, '')

    ws.append(top_row)

    for i in range(3, len(top_row), 2):
        ws.merge_cells(start_row=1, start_column=i, end_row=1, end_column=i+1)

    ws.append(next_row)

    for r in dataframe_to_rows(data, index=False, header=False):
        ws.append(r)

    export_path = f"{pathlib.Path.home()}/{filename}"

    wb.save(export_path)

    return export_path
